(*
 * Copyright (c) 2007-2011 by Evernote Corporation, All rights reserved.
 *
 * Use of the source code and binary libraries included in this package
 * is permitted under the following terms:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *)

program EDAMTest;

{$APPTYPE CONSOLE}

uses
  Classes,
  SysUtils,
  IdHashMessageDigest,
  Thrift in '..\src\Thrift\Thrift.pas',
  Thrift.Utils in '..\src\Thrift\Thrift.Utils.pas',
  Thrift.Collections in '..\src\Thrift\Thrift.Collections.pas',
  Thrift.Console in '..\src\Thrift\Thrift.Console.pas',
  Thrift.Protocol in '..\src\Thrift\Thrift.Protocol.pas',
  Thrift.Stream in '..\src\Thrift\Thrift.Stream.pas',
  Thrift.Transport in '..\src\Thrift\Thrift.Transport.pas',
  Evernote.EDAM.Error in '..\src\Evernote\Evernote.EDAM.Error.pas',
  Evernote.EDAM.Limits in '..\src\Evernote\Evernote.EDAM.Limits.pas',
  Evernote.EDAM.NoteStore in '..\src\Evernote\Evernote.EDAM.NoteStore.pas',
  Evernote.EDAM.Type_ in '..\src\Evernote\Evernote.EDAM.Type_.pas',
  Evernote.EDAM.UserStore in '..\src\Evernote\Evernote.EDAM.UserStore.pas';


function ReadFully( const FileName: string): TBytes;
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create( FileName, fmOpenRead or fmShareDenyNone);
  try
    SetLength( Result, Stream.Size);
    Stream.Position := 0;
    Stream.Read( Result[0], Stream.Size);
  finally
    Stream.Free;
  end;
end;

var
  username : string;
  password : string;
  consumerKey : string;
  consumerSecret : string;
  evernoteHost : string;
  edamBaseUrl : string;
  userStoreUrl : string;
  userStoreTransport : ITransport;
  userStoreProtocol : IProtocol;
  userStore : TUserStore.Iface;
  versionOK : Boolean;


  authResult : IAuthenticationResult;
  parameter : string;
  errorCode : TEDAMErrorCode;
  user : IUser;
  authToken : string;
  noteStoreUrl : string;
  noteStoreTransport : ITransport;
  noteStoreProtocol : IProtocol;
  noteStore : TNoteStore.Iface;

  notebook : INotebook;
  notebooks : IThriftList<INotebook>;
  defaultNotebook : INotebook;
  image : TBytes;
  hash : TBytes;
  hashHex : string;
  data : IData;
  resource : IResource;
  note : INote;
  createdNote : INote;
  md5 : TIdHashMessageDigest5;

begin
  try
    { TODO -oUser -cConsole C : ɃR[hLqĂ }

    if (ParamCount < 2) then
    begin
      Console.WriteLine('Arguments:  <username> <password>');
      Exit;
    end;
    username := ParamStr(1);
    password := ParamStr(2);

    // NOTE: You must change the consumer key and consumer secret to the
    //       key and secret that you received from Evernote
    consumerKey := 'en-edamtest';
    consumerSecret := '0123456789abcdef';

    evernoteHost := 'sandbox.evernote.com';
    edamBaseUrl := 'https://' + evernoteHost;
    // If using Mono, see http://www.mono-project.com/FAQ:_Security

    userStoreUrl := edamBaseUrl + '/edam/user';
    userStoreTransport := THTTPClientImpl.Create(userStoreUrl);
    userStoreProtocol := TBinaryProtocolImpl.Create(userStoreTransport);
    userStore := TUserStore.TClient.Create(userStoreProtocol);

    versionOK :=
        userStore.checkVersion('Delphi EDAMTest',
        Evernote.EDAM.UserStore.TConstants.EDAM_VERSION_MAJOR,
        Evernote.EDAM.UserStore.TConstants.EDAM_VERSION_MINOR);
    Console.WriteLine('Is my EDAM protocol version up to date? ' + BoolToStr( versionOK, True));
    if (not versionOK) then
    begin
      Exit;
    end;

    authResult := nil;
    try
      authResult := userStore.authenticate(username, password,
        consumerKey, consumerSecret);
    except
      on ex: TEDAMUserException do
      begin
        parameter := ex.Parameter;
        errorCode := ex.ErrorCode;

        Console.WriteLine('Authentication failed (parameter: ' + parameter +
            ' errorCode: ' + IntToStr( Integer( errorCode)) + ')');

        if (errorCode = TEDAMErrorCode.INVALID_AUTH) then
        begin
          if (parameter = 'consumerKey') then
          begin
            if (consumerKey = 'en-edamtest') then
            begin
              Console.WriteLine('You must replace the variables consumerKey and consumerSecret with the values you received from Evernote.');
            end else
            begin
              Console.WriteLine('Your consumer key was not accepted by ' + evernoteHost);
              Console.WriteLine('This sample client application requires a client API key. If you requested a web service API key, you must authenticate using OAuth');
            end;
            Console.WriteLine('If you do not have an API Key from Evernote, you can request one from http://www.evernote.com/about/developer/api');
          end else
          if (parameter = 'username') then
          begin
            Console.WriteLine('You must authenticate using a username and password from ' + evernoteHost);
            if (evernoteHost <> 'www.evernote.com') then
            begin
              Console.WriteLine('Note that your production Evernote account will not work on ' + evernoteHost + ',');
              Console.WriteLine('you must register for a separate test account at https://' + evernoteHost + '/Registration.action');
            end;
          end else
          if (parameter = 'password') then
          begin
            Console.WriteLine('The password that you entered is incorrect');
          end;
        end;
        Exit;
      end;
    end;

    user := authResult.User;
    authToken := authResult.AuthenticationToken;
    Console.WriteLine('Authentication successful for: ' + user.Username);
    Console.WriteLine('Authentication token = ' + authToken);

    noteStoreUrl := edamBaseUrl + '/edam/note/' + user.ShardId;
    noteStoreTransport := THTTPClientImpl.Create(noteStoreUrl);
    noteStoreProtocol := TBinaryProtocolImpl.Create(noteStoreTransport);
    noteStore := TNoteStore.TClient.Create(noteStoreProtocol);

    notebooks := noteStore.listNotebooks(authToken);
    Console.WriteLine('Found ' + IntToStr( notebooks.Count) + ' notebooks:');
    defaultNotebook := notebooks[0];
    for notebook in notebooks do
    begin
      Console.WriteLine('  * ' + notebook.Name);
      if (notebook.DefaultNotebook) then
      begin
        defaultNotebook := notebook;
      end;
    end;

    Console.WriteLine('');
    Console.WriteLine('Creating a note in the default notebook: ' +
                      defaultNotebook.Name);
    Console.WriteLine('');

    image := ReadFully( 'enlogo.png' );

    data := TDataImpl.Create;
    data.Size := Length(image);

    md5 := TIdHashMessageDigest5.Create;
    try
      hash := md5.HashBytes( image );
      hashhex := LowerCase( md5.HashBytesAsHex( image ) );
    finally
      md5.Free;
    end;

    data := TDataImpl.Create;
    data.Size := Length( image);
    data.BodyHash := hash;
    data.Body := image;

    resource := TResourceImpl.Create;
    resource.Mime := 'image/png';
    resource.Data := data;

    note := TNoteImpl.Create;
    note.NotebookGuid := defaultNotebook.Guid;
    note.Title := 'Test note from EDAMTest.dpr';
    note.Content := '<?xml version="1.0" encoding="UTF-8"?>' +
        '<!DOCTYPE en-note SYSTEM "http://xml.evernote.com/pub/enml2.dtd">' +
        '<en-note>Here''s the Evernote logo:<br/>' +
        '<en-media type="image/png" hash="' + hashHex + '"/>' +
        '</en-note>';

    note.Resources := TThriftListImpl<IResource>.Create;
    note.Resources.Add(resource);

    createdNote := noteStore.createNote(authToken, note);
    Console.WriteLine('Successfully created new note with GUID: ' + createdNote.Guid);
  except
    on E: Exception do
    begin
      Writeln(E.ClassName, ': ', E.Message);
    end;
  end;
end.
